# -*- coding: utf-8 -*-
"""
@file
@brief Helpers around JSON
"""
import uuid
import json
from IPython.display import display_html, display_javascript


class RenderJSONRaw(object):
    """
    render JSON using javascript
    """

    def __init__(self, json_data, width="100%", height="100%", divid=None):
        """
        initialize with a JSON data

        @param  json_data       dictionary or string
        @param  width           (str) width
        @param  height          (str) height
        @param  divid           (str|None) id of the div
        """
        if isinstance(json_data, (dict, list)):
            self.json_str = json.dumps(json_data)
        else:
            self.json_str = json
        self.uuid = divid if divid else str(uuid.uuid4())
        self.width = width
        self.height = height

    def generate_html(self):
        """
        overloads method
        `_ipython_display_ <http://ipython.readthedocs.io/en/stable/config/integrating.html?highlight=Integrating%20>`_.
        """
        ht = '<div id="{}" style="height: {}; width:{};"></div>'.format(
            self.uuid, self.width, self.height)
        js = """
        require(["https://rawgit.com/caldwell/renderjson/master/renderjson.js"], function() {
        document.getElementById('%s').appendChild(renderjson(%s))
        }); """ % (self.uuid, self.json_str)
        return ht, js


class RenderJSONObj(RenderJSONRaw):
    """
    render JSON using javascript
    """

    def _ipython_display_(self):
        ht, js = self.generate_html()
        display_html(ht, raw=True)
        display_javascript(js, raw=True)


class RenderJSON(RenderJSONRaw):
    """
    render JSON using javascript, outputs only HTML
    """

    def _repr_html_(self):
        ht, js = self.generate_html()
        ht += "\n<script>\n{0}\n</script>\n".format(js)
        return ht


def JSONJS(data, html_only=True):
    """
    Inspired from `Pretty JSON Formatting in IPython Notebook <http://stackoverflow.com/questions/18873066/pretty-json-formatting-in-ipython-notebook>`_.

    @param      data       dictionary or json string
    @return                @see cl RenderJSON

    The function uses librairy
    `renderjson <https://github.com/caldwell/renderjson>`_.
    It returns an object with overwrite method
    `_ipython_display_ <http://ipython.readthedocs.io/en/stable/config/integrating.html?highlight=Integrating%20>`_.

    .. faqref::
        :title: Persistent javascript in a conververted notebook

        After a couple of tries, it appears that it is more efficient to
        render the javascript inside a section ``<script>...</script>``
        when the notebook is converted to RST.
    """
    if html_only:
        return RenderJSON(data)
    else:
        return RenderJSONObj(data)



#-*- coding: utf-8 -*-
import sys
import os
import datetime
import re
import better

sys.path.insert(0, os.path.abspath(os.path.join(os.path.split(__file__)[0])))
sys.path.insert(
    0,
    os.path.abspath(
        os.path.join(
            os.path.split(__file__)[0],
            "..",
            "..",
            "..",
            "..",
            "pyquickhelper",
            "src")))

local_template = os.path.join(os.path.abspath(
    os.path.dirname(__file__)), "phdoc_templates")

from pyquickhelper.helpgen.default_conf import set_sphinx_variables, get_default_stylesheet
set_sphinx_variables(__file__, "jyquickhelper", "Xavier Dupré", 2017,
                     "better", [better.better_theme_path],
                     locals(), extlinks=dict(
                         issue=('https://github.com/sdpython/jyquickhelper/issues/%s', 'issue')),
                     title="jyquickhelper", book=True)

blog_root = "http://www.xavierdupre.fr/app/jyquickhelper/helpsphinx/"

html_context = {
    'css_files': get_default_stylesheet() + ['_static/my-styles.css'],
}

html_logo = "project_ico.png"

html_sidebars = {}

language = "en"
custom_preamble = """\n
\\newcommand{\\vecteur}[2]{\\pa{#1,\\dots,#2}}
\\newcommand{\\N}[0]{\\mathbb{N}}
\\newcommand{\\indicatrice}[1]{\\mathbf{1\\!\\!1}_{\\acc{#1}}}
\\usepackage[all]{xy}
\\newcommand{\\infegal}[0]{\\leqslant}
\\newcommand{\\supegal}[0]{\\geqslant}
\\newcommand{\\ensemble}[2]{\\acc{#1,\\dots,#2}}
\\newcommand{\\fleche}[1]{\\overrightarrow{ #1 }}
\\newcommand{\\intervalle}[2]{\\left\\{#1,\\cdots,#2\\right\\}}
\\newcommand{\\loinormale}[2]{{\\cal N}\\pa{#1,#2}}
\\newcommand{\\independant}[0]{\\;\\makebox[3ex]{\\makebox[0ex]{\\rule[-0.2ex]{3ex}{.1ex}}\\!\\!\\!\\!\\makebox[.5ex][l]{\\rule[-.2ex]{.1ex}{2ex}}\\makebox[.5ex][l]{\\rule[-.2ex]{.1ex}{2ex}}} \\,\\,}
\\newcommand{\\esp}{\\mathbb{E}}
\\newcommand{\\var}{\\mathbb{V}}
\\newcommand{\\pr}[1]{\\mathbb{P}\\pa{#1}}
\\newcommand{\\loi}[0]{{\\cal L}}
\\newcommand{\\vecteurno}[2]{#1,\\dots,#2}
\\newcommand{\\norm}[1]{\\left\\Vert#1\\right\\Vert}
\\newcommand{\\dans}[0]{\\rightarrow}
\\newcommand{\\partialfrac}[2]{\\frac{\\partial #1}{\\partial #2}}
\\newcommand{\\partialdfrac}[2]{\\dfrac{\\partial #1}{\\partial #2}}
\\newcommand{\\loimultinomiale}[1]{{\\cal M}\\pa{#1}}
\\newcommand{\\trace}[1]{tr\\pa{#1}}
\\newcommand{\\abs}[1]{\\left|#1\\right|}
"""
#\\usepackage{eepic}

imgmath_latex_preamble += custom_preamble
latex_elements['preamble'] += custom_preamble
mathdef_link_only = True



"""
@brief      test log(time=2s)
"""

import sys
import os
import unittest


try:
    import src
except ImportError:
    path = os.path.normpath(
        os.path.abspath(
            os.path.join(
                os.path.split(__file__)[0],
                "..",
                "..")))
    if path not in sys.path:
        sys.path.append(path)
    import src

try:
    import pyquickhelper as skip_
except ImportError:
    path = os.path.normpath(
        os.path.abspath(
            os.path.join(
                os.path.split(__file__)[0],
                "..",
                "..",
                "..",
                "pyquickhelper",
                "src")))
    if path not in sys.path:
        sys.path.append(path)
    import pyquickhelper as skip_

from pyquickhelper.loghelper import fLOG
from src.jyquickhelper import JSONJS


class TestJsonHelper(unittest.TestCase):

    def test_json_helper(self):
        fLOG(
            __file__,
            self._testMethodName,
            OutputPrint=__name__ == "__main__")

        f = JSONJS(dict(a="a"))
        assert f
        if hasattr(f, "_ipython_display_"):
            f._ipython_display_()
        else:
            f._repr_html_()

        f = JSONJS(dict(a="a"), html_only=True)
        assert f
        if hasattr(f, "_ipython_display_"):
            f._ipython_display_()
        else:
            f._repr_html_()

    def test_json_helper_api(self):
        fLOG(
            __file__,
            self._testMethodName,
            OutputPrint=__name__ == "__main__")

        import requests
        data_json = requests.get(
            "http://api.worldbank.org/countries?incomeLevel=LMC&format=json").json()
        f = JSONJS(data_json, html_only=True)
        assert f
        if hasattr(f, "_ipython_display_"):
            f._ipython_display_()
        else:
            fLOG(f._repr_html_())


if __name__ == "__main__":
    unittest.main()



# -*- coding: utf-8 -*-
"""
@file
@brief Functions to call from the notebook
"""
from IPython.display import Javascript, HTML


def store_notebook_path(name="theNotebook"):
    """
    Return Javascript object to execute in order to store
    the notebook file name into a variable
    available from the notebook.

    @param      name        name of the variable
    @return                 Javascript object

    The function uses the following code to get the notebook path::

        var kernel = IPython.notebook.kernel;
        var body = document.body,
        attribs = body.attributes;
        var command = "theNotebook = os.path.join(" + "r'"+attribs['data-project'].value+"'," +
                      "r'"+attribs['data-notebook-path'].value+"'," + "r'"+attribs['data-notebook-name'].value+"')";
        kernel.execute(command);

    Example::

        from pyquickhelper.ipythonhelper import store_notebook_path
        store_notebook_path()

    In another cell::

        theNotebook

    See notebook :ref:`exempleoffixmenurst`.
    Try function @see fn set_notebook_name_theNotebook if this one does not work.
    """
    js = """
        var kernel = IPython.notebook.kernel;
        var body = document.body,
        attribs = body.attributes;
        var command = "{0} = os.path.join(" + "r'"+attribs['data-project'].value+"'," +
                      "r'"+attribs['data-notebook-path'].value+"'," + "r'"+attribs['data-notebook-name'].value+"')";
        kernel.execute(command);
        """.replace("        ", "").format(name)
    return Javascript(js)


def set_notebook_name_theNotebook(name="theNotebook"):
    """
    This function must be called from the notebook
    you want to know the name. It relies on
    a javascript piece of code. It populates
    the variable ``theNotebook`` with the notebook name.

    @param      name        name of the variable to create

    This solution was found at
    `How to I get the current IPython Notebook name <http://stackoverflow.com/questions/12544056/how-to-i-get-the-current-ipython-notebook-name>`_.

    The function can be called in a cell.
    The variable ``theNotebook`` will be available in the next cells.

    Try function @see fn store_notebook_path if this one does not work.
    """
    code = """var kernel = IPython.notebook.kernel;
              var body = document.body, attribs = body.attributes;
              var command = "__NAME__ = " + "'"+attribs['data-notebook-name'].value+"'";
              kernel.execute(command);""".replace("              ", "").replace("__NAME__", name)

    def get_name():
        from IPython.core.display import Javascript, display
        display(Javascript(code))
    return get_name()


add_notebook_menu_js = """
                function repeat_indent_string(n){
                    var a = "" ;
                    for ( ; n > 0 ; --n)
                        a += "    ";
                    return a;
                }
                var update_menu_string = function(begin, lfirst, llast, sformat, send, keep_item, begin_format, end_format) {
                    var anchors = document.getElementsByClassName("section");
                    if (anchors.length == 0) {
                        anchors = document.getElementsByClassName("text_cell_render rendered_html");
                    }
                    var i,t;
                    var text_menu = begin;
                    var text_memo = "<pre>\\nlength:" + anchors.length + "\\n";
                    var ind = "";
                    var memo_level = 1;
                    var href;
                    var tags = [];
                    var main_item = 0;
                    var format_open = 0;
                    for (i = 0; i <= llast; i++)
                        tags.push("h" + i);

                    for (i = 0; i < anchors.length; i++) {
                        text_memo += "**" + anchors[i].id + "--\\n";

                        var child = null;
                        for(t = 0; t < tags.length; t++) {
                            var r = anchors[i].getElementsByTagName(tags[t]);
                            if (r.length > 0) {
                                child = r[0];
                                break;
                            }
                        }
                        if (child == null) {
                            text_memo += "null\\n";
                            continue;
                        }
                        if (anchors[i].hasAttribute("id")) {
                            // when converted in RST
                            href = anchors[i].id;
                            text_memo += "#1-" + href;
                            // passer à child suivant (le chercher)
                        }
                        else if (child.hasAttribute("id")) {
                            // in a notebook
                            href = child.id;
                            text_memo += "#2-" + href;
                        }
                        else {
                            text_memo += "#3-" + "*" + "\\n";
                            continue;
                        }
                        var title = child.textContent;
                        var level = parseInt(child.tagName.substring(1,2));

                        text_memo += "--" + level + "?" + lfirst + "--" + title + "\\n";

                        if ((level < lfirst) || (level > llast)) {
                            continue ;
                        }
                        if (title.endsWith('¶')) {
                            title = title.substring(0,title.length-1).replace("<", "&lt;")
                                         .replace(">", "&gt;").replace("&", "&amp;");
                        }
                        if (title.length == 0) {
                            continue;
                        }

                        while (level < memo_level) {
                            text_menu += end_format + "</ul>\\n";
                            format_open -= 1;
                            memo_level -= 1;
                        }
                        if (level == lfirst) {
                            main_item += 1;
                        }
                        if (keep_item != -1 && main_item != keep_item + 1) {
                            // alert(main_item + " - " + level + " - " + keep_item);
                            continue;
                        }
                        while (level > memo_level) {
                            text_menu += "<ul>\\n";
                            memo_level += 1;
                        }
                        text_menu += repeat_indent_string(level-2);
                        text_menu += begin_format + sformat.replace("__HREF__", href).replace("__TITLE__", title);
                        format_open += 1;
                    }
                    while (1 < memo_level) {
                        text_menu += end_format + "</ul>\\n";
                        memo_level -= 1;
                        format_open -= 1;
                    }
                    text_menu += send;
                    //text_menu += "\\n" + text_memo;

                    while (format_open > 0) {
                        text_menu += end_format;
                        format_open -= 1;
                    }
                    return text_menu;
                };
                var update_menu = function() {
                    var sbegin = "__BEGIN__";
                    var sformat = __FORMAT__;
                    var send = "__END__";
                    var begin_format = __BEGIN_FORMAT__;
                    var end_format = __END_FORMAT__;
                    var keep_item = __KEEP_ITEM__;
                    var text_menu = update_menu_string(sbegin, __FIRST__, __LAST__, sformat, send, keep_item,
                                                       begin_format, end_format);
                    var menu = document.getElementById("__MENUID__");
                    menu.innerHTML=text_menu;
                };
                window.setTimeout(update_menu,2000);
            """


def add_notebook_menu(menu_id="my_id_menu_nb", raw=False, format="html", header=None,
                      first_level=2, last_level=4, keep_item=None):
    """
    add javascript and HTML to the notebook which gathers all in the notebook and builds a menu

    @param      menu_id         menu_id
    @param      raw             raw HTML and Javascript
    @param      format          *html* or *rst*
    @param      header          title of the menu (None for None)
    @param      first_level     first level to consider
    @param      last_level      last level to consider
    @param      keep_item       None or integer (starts at 0), reduce the number of displayed items to 1
                                and its descendant
    @return                     HTML object

    In a notebook, it is easier to do by using a magic command
    ``%%html`` for the HTML and another one
    ``%%javascript`` for the Javascript.
    This function returns a full text with HTML and
    Javascript.

    If the format is RST, the menu can be copied/pasted in a text cell.

    On the notebook, the instruction would work::

        var anchors = document.getElementsByClassName("anchor-link");

    But it fails during the conversion from a notebook to format RST.
    """
    if keep_item is not None:
        menu_id += str(keep_item)
    html = '<div id="{0}">run previous cell, wait for 2 seconds</div>'.format(
        menu_id)

    global add_notebook_menu_js
    js = add_notebook_menu_js.replace("                ", "") \
                             .replace("__MENUID__", menu_id) \
                             .replace("__FIRST__", str(first_level)) \
                             .replace("__LAST__", str(last_level))

    full = "{0}\n<script>{1}</script>".format(html, js)
    if keep_item is None:
        keep_item = -1

    if format == "html":
        if header is not None and len(header) > 0:
            header = "<b>{0}</b>\n".format(header)
        else:
            header = ""
        full = header + \
            full.replace("__FORMAT__", """'<a href="#__HREF__">__TITLE__</a>'""") \
            .replace("__BEGIN__", "") \
            .replace("__END__", "") \
            .replace("__KEEP_ITEM__", str(keep_item)) \
            .replace("__BEGIN_FORMAT__", "'<li>'") \
            .replace("__END_FORMAT__", "'</li>'")
    elif format == "rst":
        if header is not None and len(header) > 0:
            header = "{0}\n\n".format(header)
        else:
            header = ""
        full = header + \
            full.replace("__FORMAT__", """'* [' + title + '](#' + href + ')\\n'""") \
                .replace("<ul>", "") \
                .replace("</ul>", "") \
                .replace("__BEGIN__", "<pre>\\n") \
                .replace("__END__", "</pre>\\n") \
                .replace("__KEEP_ITEM__", str(keep_item)) \
                .replace("__BEGIN_FORMAT__", "") \
                .replace("__END_FORMAT__", "")
    else:
        raise ValueError("format must be html or rst")

    if raw:
        return full
    else:
        return HTML(full)


def load_extension(name):
    """
    install an extension, checks first it exists,
    if not displays an exception with the list of them

    @param      name        extension name
    """
    return Javascript("IPython.utils.load_extensions('%s')" % name)



"""
@brief      test log(time=2s)
"""

import sys
import os
import unittest


try:
    import src
except ImportError:
    path = os.path.normpath(
        os.path.abspath(
            os.path.join(
                os.path.split(__file__)[0],
                "..",
                "..")))
    if path not in sys.path:
        sys.path.append(path)
    import src

try:
    import pyquickhelper as skip_
except ImportError:
    path = os.path.normpath(
        os.path.abspath(
            os.path.join(
                os.path.split(__file__)[0],
                "..",
                "..",
                "..",
                "pyquickhelper",
                "src")))
    if path not in sys.path:
        sys.path.append(path)
    import pyquickhelper as skip_

from pyquickhelper.loghelper import fLOG
from src.jyquickhelper import store_notebook_path, add_notebook_menu


class TestHelperInNotebook(unittest.TestCase):

    def test_store_notebook_path(self):
        fLOG(
            __file__,
            self._testMethodName,
            OutputPrint=__name__ == "__main__")
        r = store_notebook_path()
        assert r is not None

    def test_add_notebook_menu(self):
        fLOG(
            __file__,
            self._testMethodName,
            OutputPrint=__name__ == "__main__")
        r = add_notebook_menu()
        assert r is not None
        r = add_notebook_menu(format="rst")
        assert r is not None


if __name__ == "__main__":
    unittest.main()



# -*- coding: utf-8 -*-
"""
@file
@brief Helpers around JSON
"""
import uuid
from IPython.display import display_html, display_javascript


class RenderJSRaw(object):
    """
    render JS using javascript
    """

    def __init__(self, script, width="100%", height="100%", divid=None, css=None,
                 libs=None, style=None, only_html=False, div_class=None):
        """
        initialize with a JS script

        @param  script          (str) script
        @param  width           (str) width
        @param  height          (str) height
        @param  style           (str) style (added in ``<style>...</style>``)
        @param  divid           (str|None) id of the div
        @param  css             (list) list of css
        @param  libs            (list) list of dependencies
        @param  only_html       (bool) use only function `display_html <http://ipython.readthedocs.io/en/stable/api/generated/IPython.display.html?highlight=display_html#IPython.display.display_html>`_
                                and not `display_javascript <http://ipython.readthedocs.io/en/stable/api/generated/IPython.display.html?highlight=display_html#IPython.display.display_javascript>`_ to add
                                javascript to the page.
        @param  div_class       (str) class of the section ``div`` which will host the results
                                of the javascript
        """
        self.script = script
        self.uuid = divid if divid else "M" + \
            str(uuid.uuid4()).replace("-", "")
        self.width = width
        self.height = height
        self.libs = libs
        self.css = css
        self.style = style
        self.only_html = only_html
        self.div_class = div_class
        if "__ID__" not in script:
            raise ValueError(
                "The sript does not contain any string __ID__. It is replaced by the ID value in script:\n{0}".format(script))

    def generate_html(self):
        """
        overloads method
        `_ipython_display_ <http://ipython.readthedocs.io/en/stable/config/integrating.html?highlight=Integrating%20>`_.

        @return     HTML text, Javascript text
        """
        if self.style:
            style = "<style>{0}</style>".format(self.style)
        else:
            style = ""
        if self.div_class:
            divcl = ' class="{0}"'.format(self.div_class)
        else:
            divcl = ""
        dims = ""
        if self.height:
            dims += "height:{0};".format(self.height)
        if self.width:
            dims += "width:{0};".format(self.width)
        if len(dims) > 0:
            dims = ' style="{0}"'.format(dims)
        if self.css:
            css = "".join(
                '<link rel="stylesheet" href="{0}" type="text/css" />'.format(c) for c in self.css)
            ht = '<div id="{0}-css">{2}{3}<div{4} id="{0}"{1}></div></div>'.format(self.uuid,
                                                                                   dims, css, style, divcl)
        else:
            css = ""
            ht = '<div id="{0}-cont"{1}>{2}<div{3} id="{0}"></div></div>'.format(
                self.uuid, dims, style, divcl)

        script = self.script.replace("__ID__", self.uuid)
        if self.libs:
            names = []
            paths = []
            shims = {}
            args = []
            exports = []
            for l in self.libs:
                if isinstance(l, dict):
                    name = l.get("name", None)
                    if "path" in l:
                        p = l["path"]
                        if name is None:
                            name = ".".join((p.split("/")[-1]).split(".")[:-1])
                        path = ".".join(p.split(".")[:-1])
                        paths.append((name, path))
                    else:
                        raise KeyError(
                            "unable to find 'path' in {0}".format(l))
                    names.append(name)
                    args.append(name)
                    if "exports" in l:
                        if name not in shims:
                            shims[name] = {}
                        shims[name]["exports"] = l["exports"]
                        if isinstance(l["exports"], list):
                            exports.extend(l["exports"])
                        else:
                            exports.append(l["exports"])
                    if "deps" in l:
                        if name not in shims:
                            shims[name] = {}
                        shims[name]["deps"] = l["deps"]
                else:
                    names.append(l)
            if len(names) == 0:
                raise ValueError("names is empty.\nlibs={0}\npaths={1}\nshims={2}\nexports={3}".format(self.libs,
                                                                                                       paths, shims, exports))
            require = ",".join("'{0}'".format(l) for l in names)

            config = ["require.config({"]
            if len(paths) > 0:
                config.append("paths:{")
                for name, path in paths:
                    config.append("'{0}':'{1}',".format(name, path))
                config.append("},")
            if len(shims) > 0:
                config.append("shim:{")

                def vd(d):
                    rows = []
                    for k, v in sorted(d.items()):
                        rows.append("'{0}':{1}".format(
                            k, v if isinstance(v, list) else "'{0}'".format(v)))
                    return "{%s}" % ",".join(rows)
                for k, v in sorted(shims.items()):
                    config.append("'{0}':{1},".format(k, vd(v)))
                config.append("},")
            config.append("});")
            if len(config) > 2:
                prefix = "\n".join(config) + "\n"
            else:
                prefix = ""
            js = prefix + \
                """\nrequire([%s], function(%s) { %s });\n""" % (
                    require, ",".join(args), script)
        else:
            js = script
        if self.only_html:
            ht += "\n<script>\n%s\n</script>" % js
            return ht, None
        else:
            return ht, js


class RenderJSObj(RenderJSRaw):
    """
    render JS using javascript
    """

    def _ipython_display_(self):
        """
        overloads method
        `_ipython_display_ <http://ipython.readthedocs.io/en/stable/config/integrating.html?highlight=Integrating%20>`_.
        """
        ht, js = self.generate_html()
        if js is None:
            display_html(ht, raw=True)
        else:
            display_html(ht, raw=True)
            display_javascript(js, raw=True)


class RenderJS(RenderJSRaw):
    """
    render JS using javascript, only outputs HTML
    """

    def _repr_html_(self):
        """
        overloads method *_repr_html_*
        """
        ht, js = self.generate_html()
        ht += "\n<script>\n{0}\n</script>\n".format(js)
        return ht



"""
@brief      test log(time=2s)
"""

import sys
import os
import unittest


try:
    import src
except ImportError:
    path = os.path.normpath(
        os.path.abspath(
            os.path.join(
                os.path.split(__file__)[0],
                "..",
                "..")))
    if path not in sys.path:
        sys.path.append(path)
    import src

try:
    import pyquickhelper as skip_
except ImportError:
    path = os.path.normpath(
        os.path.abspath(
            os.path.join(
                os.path.split(__file__)[0],
                "..",
                "..",
                "..",
                "pyquickhelper",
                "src")))
    if path not in sys.path:
        sys.path.append(path)
    import pyquickhelper as skip_

from pyquickhelper.loghelper import fLOG
from src.jyquickhelper import RenderJS
from src.jyquickhelper.javascript_helper import RenderJSObj


class TestJavascriptHelper(unittest.TestCase):

    def test_javascript_helper(self):
        fLOG(
            __file__,
            self._testMethodName,
            OutputPrint=__name__ == "__main__")

        try:
            f = RenderJSObj("", css=["_css"], libs=["_libs"])
        except ValueError:
            pass
        f = RenderJSObj("__ID__", css=["_css"], libs=["_libs"])
        assert f
        f._ipython_display_()

        f = RenderJSObj("__ID__", css=None, libs=["_libs"])
        assert f
        f._ipython_display_()

        f = RenderJSObj("__ID__", css=None, libs=None)
        assert f
        f._ipython_display_()

        f = RenderJSObj("__ID__", css=["_css"], libs=None)
        assert f
        f._ipython_display_()

        f = RenderJSObj("__ID__", css=["_css"], libs=["_libs"], style="r")
        assert f
        f._ipython_display_()

        f = RenderJSObj("__ID__", css=["_css"], libs=[
            "_libs"], style="r", width=None, height=None)
        assert f
        f._ipython_display_()

        f = RenderJS("__ID__", css=["_css"], libs=[
                     "_libs"], style="r", width=None, height=None)
        assert f
        f._repr_html_()

    def test_javascript_helper_config(self):
        fLOG(
            __file__,
            self._testMethodName,
            OutputPrint=__name__ == "__main__")

        script = """
         var chart = c3.generate({
            bindto: '#__ID__',
            data: {
              columns: [
                ['data1', 30, 200, 100, 400, 150, 250],
                ['data2', 50, 20, 10, 40, 15, 25]
              ]
            }
        });
        """
        f = RenderJS(script, divid="MYID", css=["http://c3js.org/css/c3.css"],
                     libs=[dict(name="d3", path="http://d3js.org/d3.v3.min.js", exports="d3"),
                           dict(name="c3", path="http://c3js.org/js/c3.min-4c5bef8f.js", exports="c3", deps=["d3"])])
        assert f
        ht, js = f.generate_html()
        assert ht
        assert js
        exp = """<div id="MYID-css"><link rel="stylesheet" href="http://c3js.org/css/c3.css" type="text/css" />""" + \
              """<div id="MYID" style="height:100%;width:100%;"></div></div>"""
        assert exp in ht
        exp = """
            require.config({
            paths:{
            'd3':'http://d3js.org/d3.v3.min',
            'c3':'http://c3js.org/js/c3.min-4c5bef8f',
            },
            shim:{
            'c3':{'deps':['d3'],'exports':'c3'},
            'd3':{'exports':'d3'},
            },
            });

            require(['d3','c3'],function(d3,c3){
            varchart=c3.generate({
            bindto:'#MYID',
            data:{
            columns:[
            ['data1',30,200,100,400,150,250],
            ['data2',50,20,10,40,15,25]
            ]
            }
            });
            });
        """.replace(" ", "").replace("\r", "").strip("\n ")
        js = js.replace(" ", "").replace(
            "\r", "").strip("\n ").replace("u'", "'")
        self.maxDiff = None
        # fLOG(js)
        self.assertEqual(js, exp)


if __name__ == "__main__":
    unittest.main()
